#pragma once

#include "Betriebssystem.h"

#ifdef WIN32
struct ID3D11Buffer;
struct D3D11_BUFFER_DESC;
struct ID3D11ShaderResourceView;
#endif

namespace Framework
{
    class Render3D;

    // Eine Schnittstelle zwischen dem Arbeitsspeicher und dem Grafikspeicher
    class DXBuffer
    {
    private:
        void *data;
        bool changed;
        int altLen;
        int elLem;

    protected:
        int ref;
#ifdef WIN32
        D3D11_BUFFER_DESC *description;
        ID3D11Buffer *buffer;
#endif

    public:
        // Konstruktor
        //  bind: Der verwendungszweck des Buffers. Beispiel: D3D11_BIND_INDEX_BUFFER, D3D11_BIND_VERTEX_BUFFER.
        //  eLen: Lnge eines einzelnen Elements in Bytes
        __declspec( dllexport ) DXBuffer( int eLen );
        // Destruktor
        __declspec( dllexport ) virtual ~DXBuffer();
        // Setzt den gendert flg, so das beim nchsten auruf von 'kopieren' die daten neu kopiert werden
        __declspec( dllexport ) void setChanged();
        // ndert die lnge des Buffers beim nchsten aufruf von 'kopieren'
        //  len: Die Lnge in Bytes
        __declspec( dllexport ) void setLength( int len );
        // Legt fest, was beim nchsten aufruf von 'kopieren' kopiert wird
        //  data: Ein zeiger auf die Daten
        __declspec( dllexport ) void setData( void *data );
        // Kopiert die Daten in den Buffer, fals sie sich verndert haben
        //  zRObj: Das Objekt, mit dem die Grafikkarte angesprochen wird
        __declspec( dllexport ) virtual void copieren( Render3D *zRObj );
        // Gibt die Lnge eines Elementes in bytes zurck
        __declspec( dllexport ) int getElementLength() const;
#ifdef WIN32
        // Gibt den Buffer zurck
        __declspec( dllexport ) ID3D11Buffer *zBuffer() const;
#endif
        // Gibt die Anzahl der Elemente im Buffer zurck
        __declspec( dllexport ) int getElementAnzahl() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) DXBuffer *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) virtual DXBuffer *release();
    };

    // Ein Buffer von Eckpunkten eines 3D Models
    class DXVertexBuffer : public DXBuffer
    {
    public:
        // Konstruktor
        // eSize: Die Lnge eines Elementes in Bytes
        __declspec( dllexport ) DXVertexBuffer( int eSize );
        // Destruktor
        __declspec( dllexport ) virtual ~DXVertexBuffer();
    };

    // Ein Buffer von Indizes aus dem Buffer mit Eckpunkten, wovon immer drei ein Dreieck ergeben, das gezeichnet wird
    class DXIndexBuffer : public DXBuffer
    {
    public:
        // Konstruktor
        // eSize: Die Lnge eines Elementes in Bytes
        __declspec( dllexport ) DXIndexBuffer( int eSize );
        // Destruktor
        __declspec( dllexport ) virtual ~DXIndexBuffer();
    };
#ifdef WIN32
    // Ein Buffer von Indizes aus dem Buffer mit Eckpunkten, wovon immer drei ein Dreieck ergeben, das gezeichnet wird
    class DXStructuredBuffer : public DXBuffer
    {
    private:
        ID3D11ShaderResourceView *view;
    public:
        // Konstruktor
        // eSize: Die Lnge eines Elementes in Bytes
        __declspec( dllexport ) DXStructuredBuffer( int eSize );
        // Kopiert die Daten in den Buffer, fals sie sich verndert haben
        //  zRObj: Das Objekt, mit dem die Grafikkarte angesprochen wird
        __declspec( dllexport ) void copieren( Render3D *zRObj ) override;
        // Gibt die verwendtete Shader Resource View zurck
        __declspec( dllexport ) operator ID3D11ShaderResourceView *() const;
        // Destruktor
        __declspec( dllexport ) virtual ~DXStructuredBuffer();
    };
#endif
}